/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<octave/ov.h>
#include<octave/ov-struct.h> /* for octave-struct */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>


using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}


class FLTKWrapper : public octave_struct
{
public:
  string *classname;
  void * pointer;
  bool isstatic;

 public: 
  FLTKWrapper()
  {
    classname=NULL;
    pointer=(void *)0;
    isstatic=false;
  }
  
  /* Well the constructor!! */
  FLTKWrapper(void *ptr,string class_na,bool staticflag=false)
  {
    std::cout<<"Wrapper type "<<class_na<<" initialized"<<std::endl;
    pointer=ptr;
    classname=new string(class_na);
    isstatic=staticflag;
  }





  ~FLTKWrapper(void)
    {
      /*std::cout<<"Your Object handle is invalidated"<<std::endl;*/
      if( classname != NULL)
	  delete classname;
    }  

  octave_value_list subsref(const std::string & str,
		       const std::list<octave_value_list>&  idx,
		       int nargout)
  {    
    octave_value_list ovl=idx.front();
    std::cout<<"subsref operator called with #"<<str<<"# "<<std::endl;
    for(int i=0;i<ovl.length();i++)
      {
	std::cout<<i<<" ";
	ovl(i).print(std::cout,true);
	std::cout<<"\n"<<std::endl;
      }
    std::cout<<"Entering evaluator"<<std::endl;

    if(str[0]=='.')
      {
	octave_value_list args;
	int offset=0;
	//std::cout<<"Ready to Jump String "<<*this->classname<<std::endl;
	std::string func_name (*this->classname +"_" + ovl(0).string_value());
	//std::cout<<"Ready to Jump List"<<std::endl;
	std::list<octave_value_list>::const_iterator pidx = idx.begin();
	//std::cout<<"Ready to Jump Eval"<<std::endl;

	// Static methods donot need a pointer to function.
	if(isstatic)	  
	    offset=0;
	else
	  {
	    // Member-methods *DO* need a pointer to function.
	    args(0)=(long int)this->pointer;
	    offset=1;
	  }
	
	if(pidx!=NULL)
	  {
	    pidx++;
	    for(int i=0;i<(*pidx).length();i++)
	      {
		args(i+offset)=(*pidx)(i);
	      }
	  }
	  
	std::cout<<"Evaluating: "<<func_name<<std::endl;

	return feval(func_name,args,nargout);
      }
    return octave_value_list();
  }

  DECLARE_OCTAVE_ALLOCATOR
  DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
};

DEFINE_OCTAVE_ALLOCATOR (FLTKWrapper);
DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (FLTKWrapper,"FLTKWrapper","FLTKWrapper");


static void print_args(octave_value_list args)
{
  for(int i=0;i<args.length();i++)
    {
      args(i).print(std::cout);
    }
  return;
}

/* static function , so must not override */
DEFUN_DLD( Fl_Window , args , , " Fl_Window * Fl_Window (int w,int h); " )
{
Fl_Window *   _fltk ;
int   w ;
int   h ;
 print_args(args); 
 if(args.length()!= 2)
  {
   std::cout<< "eg:  Fl_Window * Fl_Window (int w,int h); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Window "<<std::endl;
#endif /* DEBUG */
w = (  int  ) args( 0 ).int_value();
h = (  int  ) args( 1 ).int_value();
 _fltk = new Fl_Window ( w,h);
return octave_value(new FLTKWrapper(_fltk,"Fl_Window"));
}
DEFUN_DLD( Fl_Box , args , , " Fl_Box * Fl_Box (int box_type,int w,int h,int x,int y,char * title); " )
{
Fl_Box *   _fltk ;
Fl_Boxtype   box_type ;
int   w ;
int   h ;
int   x ;
int   y ;
char *   title ;
 if(args.length()!= 6 )
  {
   std::cout<< "eg:  Fl_Box * Fl_Box (int box_type,int w,int h,int x,int y,char * title); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Box "<<std::endl;
#endif /* DEBUG */
box_type = (Fl_Boxtype ) args( 0 ).int_value();
w = (  int  ) args( 1 ).int_value();
h = (  int  ) args( 2 ).int_value();
x = (  int  ) args( 3).int_value();
y = (  int  ) args( 4 ).int_value();
title = new  char   [args( 5 ).string_value ().length()];
strcpy( (char *) title ,args( 5 ).string_value().c_str());
_fltk = new Fl_Box (box_type,w,h,x,y,title);
return octave_value(new FLTKWrapper(_fltk,"Fl_Box"));
}
DEFUN_DLD( Fl_Box_labelfont , args , , " void Fl_Box_labelfont (int type); " )
{
Fl_Box *   _fltk ;
int   type ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  void Fl_Box_labelfont (int type); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Box_labelfont "<<std::endl;
#endif /* DEBUG */
_fltk =( Fl_Box * )args( 0 ).long_value();
type = (  int  ) args( 1 ).int_value();
_fltk->labelfont ( type);
return octave_value();
}
DEFUN_DLD( Fl_Box_labelsize , args , , " void Fl_Box_labelsize (int val); " )
{
Fl_Box *   _fltk ;
int   val ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  void Fl_Box_labelsize (int val); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
 std::cout<<" Fl_Box_labelsize "<<std::endl;
#endif /* DEBUG */
 _fltk =(  Fl_Box* )args( 0 ).long_value();
 val = (  int  ) args( 1 ).int_value();
 _fltk->labelsize ( val);
return octave_value();
}
DEFUN_DLD( Fl_Box_labeltype , args , , " void Fl_Box_labeltype (int type); " )
{
Fl_Box *   _fltk ;
 Fl_Labeltype   type ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  void Fl_Box_labeltype (int type); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Box_labeltype "<<std::endl;
#endif /* DEBUG */
_fltk =( Fl_Box * )args( 0 ).long_value();
 type = (  Fl_Labeltype  ) args( 1 ).int_value();
 _fltk->labeltype (type);
return octave_value();
}

/* static function */
DEFUN_DLD( Fl_Window_end , args , , " void Fl_Window_end ( ); " )
{
  Fl_Window *window;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  void Fl_Window_end ( ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
 std::cout<<" Fl_Window_end "<<std::endl;
#endif /* DEBUG */
/* Nothing! */
 window=(Fl_Window *)args(0).long_value();
 window->end();
 return octave_value();
}

DEFUN_DLD( Fl_Window_show , args , , 
	   " void Fl_Window_show (Cell []); " )
{
  Fl_Window *   _fltk ;
  Cell  matrix;

  if(args.length()!= 2)
    {
      std::cout<< "eg:  void Fl_Window_show (Cell []); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Window_show "<<std::endl;
#endif /* DEBUG */


  _fltk =( Fl_Window * )args( 0 ).long_value();
  matrix=args(1).cell_value();
  char *list_args[matrix.cols()];

  //matrix.print(std::cout,true);
  std::cout<<"r= "<<matrix.rows()<<"c= "<<matrix.cols()<<std::endl;

  for(int i=0;i<matrix.cols();i++)
    {
      list_args[i]=strdup(matrix(0,i).string_value().c_str());
      matrix(0,i).print(std::cout,true);
    }
  
  if (matrix.cols()<= 0)
    {
      char *val[]={"unknown",NULL};  
      _fltk->show(1,val);
    }
  else
    _fltk->show(matrix.cols(),list_args);
  
  
  for(int i=0;i<matrix.cols();i++)
    free(list_args[i]);

  _fltk->show(0,NULL);

  return octave_value();
}

/* static function */
DEFUN_DLD( Fl_run , args , , " void Fl_run ( ); " )
{
#ifdef DEBUG
std::cout<<" Fl_run "<<std::endl;
#endif /* DEBUG */
 Fl::run ( );
return octave_value();
}
/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( fltk , args , , " void fltk(void); " )
{
  /* some enumerations you will want */
  DEFCONST_INTERNAL(FL_UP_BOX,FL_UP_BOX,"");
  DEFCONST_INTERNAL(FL_BOLD,FL_BOLD,"");
  DEFCONST_INTERNAL(FL_ITALIC,FL_ITALIC,"");
  DEFCONST_INTERNAL(FL_SHADOW_LABEL,FL_SHADOW_LABEL,"");
  
  octave_dynamic_loader::load("Fl_Window" ,"");
  octave_dynamic_loader::load("Fl_Box" ,"");
  octave_dynamic_loader::load("Fl_Box_labelfont" ,"");
  octave_dynamic_loader::load("Fl_Box_labelsize" ,"");
  octave_dynamic_loader::load("Fl_Box_labeltype" ,"");
  octave_dynamic_loader::load("Fl_Window_end" ,"");
  octave_dynamic_loader::load("Fl_Window_show" ,"");
  octave_dynamic_loader::load("Fl_run" ,"");
  std::cout<<"Code generated onSat Jul 30 16:33:58 2005\n Compiled on " __DATE__  __TIME__ "\n loaded fltkLibrary!";

  return octave_value(new FLTKWrapper(0,"Fl",true));
}
